local MSQ = LibStub and LibStub("Masque", true)
if not MSQ then return end

local ADDON = ...

local VERSION = "1.0.4"
local MASQUE_VERSION = 90002

local path = [[Interface\AddOns\]] .. ADDON .. [[\Textures\]]

local scale = function(contentSize, sourceTextureSize)
	return (sourceTextureSize or contentSize)/contentSize * 36
end

MSQ:AddSkin("Retina", {
	Shape = "Square",
	Masque_Version = MASQUE_VERSION,
	Description = "High resolution, pixel perfect skin that blends in well with the default Blizzard UI",
	Version = VERSION,
	Author = "|cff6F85D5jmxd#9366|r",
	Title = "Retina",
	Order = 1,

	Backdrop = {
		Texture = path .. "Backdrop.tga",
		Color = {1, 1, 1, 1},
		BlendMode = "BLEND",
		DrawLayer = "BACKGROUND",
		DrawLevel = -1,
		Width = scale(72,128),
		Height = scale(72,128),
		Point = "CENTER",
		RelPoint = "CENTER",
		OffsetX = 0,
		OffsetY = 0
	},
	Icon = {
		TexCoords = { 5/72, 67/72, 5/72, 67/72 },
		DrawLayer = "BACKGROUND",
		DrawLevel = 0,
		Width = scale(72,66),
		Height = scale(72,66),
		Point = "CENTER",
		RelPoint = "CENTER",
		OffsetX = 0,
		OffsetY = 0,
	},
	Cooldown = {
		Color = {0, 0, 0, 0.7},
		Width = scale(70,60),
		Height = scale(70,60),
		Point = "CENTER",
		RelPoint = "CENTER",
		OffsetX = 0,
		OffsetY = 0,
	},
	Flash = {
		Texture = path .. "Border.tga",
		Color = {1, 0, 0, 0.75},
		BlendMode = "BLEND",
		DrawLayer = "OVERLAY",
		DrawLevel = 2,
		Width = scale(72,128),
		Height = scale(72,128),
		Point = "CENTER",
		RelPoint = "CENTER",
		OffsetX = 0,
		OffsetY = 0
	},
	Pushed = {
		Texture = path .. "Border.tga",
		Color = {0, .68, 1, 1},
		BlendMode = "BLEND",
		DrawLayer = "OVERLAY",
		DrawLevel = 1,
		Width = scale(72,128),
		Height = scale(72,128),
		Point = "CENTER",
		RelPoint = "CENTER",
		OffsetX = 0,
		OffsetY = 0
	},
	Normal = {
		Texture = path .. "Normal.tga",
		Color = { .6, .6, .6, 1 },
		BlendMode = "BLEND",
		DrawLayer = "ARTWORK",
		DrawLevel = 0,
		Width = scale(72,128),
		Height = scale(72,128),
		Point = "CENTER",
		RelPoint = "CENTER",
		OffsetX = 0,
		OffsetY = 0,
	},
	Checked = {
		Texture = path .. "Checked.tga",
		Color = { 0, .68, 1, 1 },
		BlendMode = "BLEND",
		DrawLayer = "OVERLAY",
		DrawLevel = 0,
		Width = scale(72,128),
		Height = scale(72,128),
		Point = "CENTER",
		RelPoint = "CENTER",
		OffsetX = 0,
		OffsetY = 0
	},
  SlotHighlight = {
    Texture = path .. "Checked.tga",
    Color = { 0, .68, 1, 1 },
    BlendMode = "BLEND",
    DrawLayer = "OVERLAY",
    DrawLevel = 0,
    Width = scale(72,128),
    Height = scale(72,128),
    Point = "CENTER",
    RelPoint = "CENTER",
    OffsetX = 0,
    OffsetY = 0
  },
	Border = {
		Texture = path .. "Border.tga",
		Color = { 1, 1, 1, 1 },
		BlendMode = "BLEND",
		DrawLayer = "OVERLAY",
		DrawLevel = 0,
		Width = scale(72,128),
		Height = scale(72,128),
		Point = "CENTER",
		RelPoint = "CENTER",
		OffsetX = 0,
		OffsetY = 0
	},
	AutoCastable = {
		Texture = path .. "AutoCastable.tga",
		Color = { 1, .82, .25, 1 },
		BlendMode = "BLEND",
		DrawLayer = "OVERLAY",
		DrawLevel = 0,
		Width = scale(72,128),
		Height = scale(72,128),
		Point = "CENTER",
		RelPoint = "CENTER",
		OffsetX = 0,
		OffsetY = 0
	},
	Highlight = {
		Texture = path .. "Highlight.tga",
		Color = { 1, .82, .25, 1 },
		BlendMode = "BLEND",
		DrawLayer = "HIGHLIGHT",
		DrawLevel = 0,
		Width = scale(72,128),
		Height = scale(72,128),
		Point = "CENTER",
		RelPoint = "CENTER",
		OffsetX = 0,
		OffsetY = 0
	},
	HotKey = {
		JustifyH = "RIGHT",
		JustifyV = "TOP",
		DrawLayer = "ARTWORK",
		Width = 36,
		Height = 10,
		Point = "TOPRIGHT",
		RelPoint = "TOPRIGHT",
		OffsetX = 0,
		OffsetY = -4,
	},
	Count = {
		JustifyH = "RIGHT",
		JustifyV = "BOTTOM",
		DrawLayer = "ARTWORK",
		Width = 36,
		Height = 10,
		Point = "BOTTOMRIGHT",
		RelPoint = "BOTTOMRIGHT",
		OffsetX = -1,
		OffsetY = 2,
	},
	Name = {
		JustifyH = "CENTER",
		JustifyV = "MIDDLE",
		DrawLayer = "OVERLAY",
		Width = 40,
		Height = 10,
		Point = "BOTTOM",
		RelPoint = "BOTTOM",
		OffsetX = 0,
		OffsetY = 2,
	},
	AutoCastShine = {
		Width = 32,
		Height = 32,
		Point = "CENTER",
		RelPoint = "CENTER",
		OffsetX = 1,
		OffsetY = -1,
	},
	Shadow = {
		Texture = path .. "Shadow.tga",
		Color = {0, 0, 0, 0.25},
		BlendMode = "BLEND",
		DrawLayer = "ARTWORK",
		DrawLevel = -1,
		Width = scale(72,128),
		Height = scale(72,128),
		Point = "CENTER",
		RelPoint = "CENTER",
		OffsetX = 0,
		OffsetY = 0
	},
	Gloss = {
		Texture = path .. "Gloss.tga",
		Color = {0, 0, 0, 0.5},
		BlendMode = "BLEND",
		DrawLayer = "OVERLAY",
		DrawLevel = 0,
		Width = scale(72,128),
		Height = scale(72,128),
		Point = "CENTER",
		RelPoint = "CENTER",
		OffsetX = 0,
		OffsetY = 0,
	},
	NewItem = {
		Texture = path .. "NewItem.tga",
		Color = {1, 1, 1, 1},
		BlendMode = "ADD",
		DrawLayer = "OVERLAY",
		DrawLevel = 2,
		Width = scale(72,128),
		Height = scale(72,128),
		Point = "CENTER",
		RelPoint = "CENTER",
		OffsetX = 0,
		OffsetY = 0,
	},
	QuestBorder = {
		Border = path .. "Border.tga",
		Texture = path .. "Quest.tga",
		Color = { 1, 1, 1, 1 },
		BlendMode = "BLEND",
		DrawLayer = "OVERLAY",
		DrawLevel = 5,
		Width = scale(72,128),
		Height = scale(72,128),
		Point = "CENTER",
		RelPoint = "CENTER",
		OffsetX = 0,
		OffsetY = 0,
	},
	NewAction = {
		Texture = path .. "NewItem.tga",
		Color = { 1, .82, .25, 1 },
		BlendMode = "ADD",
		DrawLayer = "OVERLAY",
		DrawLevel = 2,
		Width = scale(72,128),
		Height = scale(72,128),
		Point = "CENTER",
		RelPoint = "CENTER",
		OffsetX = 0,
		OffsetY = 0,
	},
	UpgradeIcon = {
		Texture = path .. "Upgrade.tga",
		BlendMode = "BLEND",
		DrawLayer = "OVERLAY",
		DrawLevel = 2,
		Width = scale(72,128),
		Height = scale(72,128),
		Point = "CENTER",
		RelPoint = "CENTER",
		OffsetX = 0,
		OffsetY = 0,
	},
	JunkIcon = {
		Texture = path .. "Junk.tga",
		BlendMode = "BLEND",
		DrawLayer = "OVERLAY",
		DrawLevel = 2,
		Width = scale(72,128),
		Height = scale(72,128),
		Point = "CENTER",
		RelPoint = "CENTER",
		OffsetX = 0,
		OffsetY = 0,
	}
}, true)

MSQ:AddSkin("Retina Action Bar", {
  Template = "Retina",
  Title = "Retina Action Bar",
  Order = 2,

  Backdrop = {
    Texture = path .. "BackdropActionBar.tga",
    Color = { 0, 0, 0, .5 },
    BlendMode = "BLEND",
    DrawLayer = "BACKGROUND",
    DrawLevel = -1,
    Width = scale(72,128),
    Height = scale(72,128),
    Point = "CENTER",
    RelPoint = "CENTER",
    OffsetX = 0,
    OffsetY = 0
  },
  Normal = {
    Texture = path .. "Normal.tga",
    Color = { .6, .6, .6, 1 },
    EmptyTexture = path .. "EmptyNormal.tga",
    EmptyColor = { .6, .6, .6, 1 },
    BlendMode = "BLEND",
    DrawLayer = "ARTWORK",
    DrawLevel = 0,
    Width = scale(72,128),
    Height = scale(72,128),
    Point = "CENTER",
    RelPoint = "CENTER",
    OffsetX = 0,
    OffsetY = 0
  },
}, true)

MSQ:AddSkin("Retina Framed", {
	Template = "Retina",
	Title = "Retina Framed",
	Order = 3,

	Backdrop = {
		Texture = path .. "BackdropFramed.tga",
		Color = {1, 1, 1, 1},
		BlendMode = "BLEND",
		DrawLayer = "BACKGROUND",
		DrawLevel = -1,
		Width = scale(72,128),
		Height = scale(72,128),
		Point = "CENTER",
		RelPoint = "CENTER",
		OffsetX = 0,
		OffsetY = 0
	},
  Normal = {
    Texture = path .. "Normal.tga",
    Color = { .6, .6, .6, 1 },
    EmptyTexture = path .. "EmptyNormal.tga",
    EmptyColor = { .6, .6, .6, 1 },
    BlendMode = "BLEND",
    DrawLayer = "ARTWORK",
    DrawLevel = 0,
    Width = scale(72,128),
    Height = scale(72,128),
    Point = "CENTER",
    RelPoint = "CENTER",
    OffsetX = 0,
    OffsetY = 0
  },
}, true)